package gov.usgs.earthquake.nshmp.www;

import java.nio.file.Path;
import java.util.concurrent.Executors;

import org.crac.Context;
import org.crac.Resource;

import software.amazon.awssdk.services.ssm.SsmClient;
import software.amazon.awssdk.services.ssm.model.ParameterType;
import software.amazon.awssdk.services.ssm.model.PutParameterRequest;
import software.amazon.awssdk.services.ssm.model.SsmException;

import com.google.common.util.concurrent.MoreExecutors;

import gov.usgs.earthquake.nshmp.gmm.Gmm;

import io.micronaut.context.annotation.Value;
import io.micronaut.crac.OrderedResource;
import jakarta.inject.Singleton;

/**
 * Prime application for AWS Lambda snapstart.
 */
@Singleton
public class PrimingResource implements OrderedResource {

  @Value("${nshmp-haz.model-path}")
  private Path modelPath;

  @Override
  public void beforeCheckpoint(Context<? extends Resource> context) throws Exception {
    ServletUtil.model(ServletUtil.loadModel(modelPath));
    updateParameter();
  }

  @Override
  public void afterRestore(Context<? extends Resource> context) throws Exception {
    ServletUtil.CALC_EXECUTOR.shutdown();
    ServletUtil.TASK_EXECUTOR.shutdown();
    ServletUtil.CALC_EXECUTOR =
        MoreExecutors.listeningDecorator(Executors.newFixedThreadPool(ServletUtil.THREAD_COUNT));
    ServletUtil.TASK_EXECUTOR = Executors.newSingleThreadExecutor();
  }

  private void updateParameter() throws SsmException {
    var parameterName = System.getenv("IMT_PARAMETER_NAME");

    if (parameterName == null) {
      throw new RuntimeException("IMT_PARAMETER_NAME not found");
    }

    var client = SsmClient.create();

    var parameterRequest = PutParameterRequest.builder()
        .name(parameterName)
        .type(ParameterType.STRING)
        .value(ServletUtil.GSON2.toJson(Gmm.supportedImts(ServletUtil.model().gmms())))
        .overwrite(true)
        .build();

    client.putParameter(parameterRequest);
  }
}
